/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-rcbsock-0.c,v 1.5 2013/09/12 16:03:03 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/test.h"
#include "sm/net.h"
#include "sm/rcb.h"

#include <stdio.h>

#define INITLEN	64
#define SMAXLEN	256
#define RT	0x0200101a

static void
test_harness(sm_rpool_P a)
{
	uint32_t len, ui;
	sm_rcb_P s;
	sm_ret_T r;
	sm_sockaddr_T sockaddr, sa4;
	sm_ipaddr_T ipaddr, ipa4;
#if MTA_NETINET6
	sm_sockaddr_T sockaddr6, sa6;
	sm_ipaddr_T ipaddr6, ipa6;
#endif

	sm_memzero(&sockaddr, sizeof(sockaddr));
	sm_memzero(&sa4, sizeof(sa4));
	sm_memzero(&ipaddr, sizeof(ipaddr));
	sm_memzero(&ipa4, sizeof(ipa4));
#if MTA_NETINET6
	sm_memzero(&sockaddr6, sizeof(sockaddr6));
	sm_memzero(&sa6, sizeof(sa6));
	sm_memzero(&ipaddr6, sizeof(ipaddr6));
	sm_memzero(&ipa6, sizeof(ipa6));
#endif

	s = NULL;

	sockaddr.sa.sa_family = AF_INET;
	r = inet_aton("1.23.45.255", &sockaddr.sin.sin_addr);
	SM_TEST_E(1 == r);
	sockaddr.sin.sin_port = 12;

	ipaddr.ipa_family = AF_INET;
	ipaddr.ipa_addr.ipv4_addr = sockaddr.sin.sin_addr.s_addr;

#if MTA_NETINET6
	sockaddr6.sa.sa_family = AF_INET6;
	r = inet_pton(AF_INET6, "1234:5678:9abc:def0:fedc:dead:f00f:0101",
			&sockaddr6.sin6.sin6_addr);
	SM_TEST_E(1 == r);
	sockaddr6.sin6.sin6_port = 9999;
	ipaddr6.ipa_family = AF_INET6;
	IPV6_ASSIGN(&ipaddr6.ipa_addr.ipv6_addr, &sockaddr6.sin6.sin6_addr);
#endif

	s = sm_rcb_new(a, INITLEN, SMAXLEN);
	SM_TEST_E(s != NULL);

	r = sm_rcb_putrec(s, RCB_PUTR_DFLT, 0, SMAXLEN,
			SM_RCBV_SOCK, RT, &sockaddr,
			SM_RCBV_IPADDR, RT, &ipaddr,
#if MTA_NETINET6
			SM_RCBV_SOCK, RT, &sockaddr6,
			SM_RCBV_IPADDR, RT, &ipaddr6,
#endif
			SM_RCBV_END);
	SM_TEST_E(SM_SUCCESS == r);

	r = sm_rcb_open_dec(s);
	SM_TEST_E(SM_SUCCESS == r);
	r = sm_rcb_getuint32(s, &ui);	/* total record size */
	SM_TEST(SM_SUCCESS == r);

	r = sm_rcb_get2uint32(s, &len, &ui);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST_E(RT == ui);

	r = sm_rcb_getsock(s, &sa4);
	SM_TEST_E(SM_SUCCESS == r);

	SM_TEST(sa4.sa.sa_family == AF_INET);
	SM_TEST(sa4.sin.sin_port == sockaddr.sin.sin_port);
	SM_TEST(sa4.sin.sin_addr.s_addr == sockaddr.sin.sin_addr.s_addr);

	r = sm_rcb_get2uint32(s, &len, &ui);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST_E(RT == ui);
	r = sm_rcb_getipaddr(s, &ipa4);
	SM_TEST_E(SM_SUCCESS == r);

#if MTA_NETINET6
	r = sm_rcb_get2uint32(s, &len, &ui);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST_E(RT == ui);
	r = sm_rcb_getsock(s, &sa6);
	SM_TEST_E(SM_SUCCESS == r);

	SM_TEST(sa6.sa.sa_family == AF_INET6);
	SM_TEST(sa6.sin6.sin6_port == sockaddr6.sin6.sin6_port);
	SM_TEST(sm_memeq(&sa6.sin6.sin6_addr, &sockaddr6.sin6.sin6_addr,
			sizeof(sockaddr6.sin6.sin6_addr)));

	r = sm_rcb_get2uint32(s, &len, &ui);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST_E(RT == ui);
	r = sm_rcb_getipaddr(s, &ipa6);
	SM_TEST_E(SM_SUCCESS == r);

#endif

	SM_TEST(SM_RCB_ISEOB(s));
	r = sm_rcb_close_dec(s);
	SM_TEST_E(SM_SUCCESS == r);

/* write 2 */
	r = sm_rcb_open_enc(s, -1);
	SM_TEST_E(SM_SUCCESS == r);
	r = sm_rcb_putuint32(s, 0);	/* dummy for record size */
	SM_TEST_E(SM_SUCCESS == r);
	r = sm_rcb_putipaddr(s, RT, &ipaddr);
	SM_TEST_E(SM_SUCCESS == r);
#if MTA_NETINET6
	r = sm_rcb_putipaddr(s, RT, &ipaddr6);
	SM_TEST_E(SM_SUCCESS == r);
#endif

	(void) sm_rcb_close_enc(s);

/* read 2 */
	r = sm_rcb_open_dec(s);
	SM_TEST_E(SM_SUCCESS == r);
	r = sm_rcb_getuint32(s, &ui);	/* total record size */
	SM_TEST(SM_SUCCESS == r);

	r = sm_rcb_get2uint32(s, &len, &ui);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST_E(RT == ui);
	r = sm_rcb_getipaddr(s, &ipa4);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST(sa4.sa.sa_family == AF_INET);
	SM_TEST(sa4.sin.sin_port == sockaddr.sin.sin_port);
	SM_TEST(sa4.sin.sin_addr.s_addr == sockaddr.sin.sin_addr.s_addr);

#if MTA_NETINET6
	r = sm_rcb_get2uint32(s, &len, &ui);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST_E(RT == ui);
	r = sm_rcb_getipaddr(s, &ipa6);
	SM_TEST_E(SM_SUCCESS == r);
	SM_TEST(sa6.sa.sa_family == AF_INET6);
	SM_TEST(sa6.sin6.sin6_port == sockaddr6.sin6.sin6_port);
	SM_TEST(sm_memeq(&sa6.sin6.sin6_addr, &sockaddr6.sin6.sin6_addr,
			sizeof(sockaddr6.sin6.sin6_addr)));
#endif

	SM_TEST(SM_RCB_ISEOB(s));
	r = sm_rcb_close_dec(s);
	SM_TEST_E(SM_SUCCESS == r);



  error:
	SM_RCB_FREE(s);
}

int
main(int argc, char *argv[])
{
	sm_rpool_P a;

	sm_test_begin(argc, argv, "test rcbsock 0");

	/* create an rpool for entire test */
	a = sm_rpool_new(NULL);
	SM_TEST(a != NULL);
	test_harness(a);
	sm_rpool_delete(a);

	/* test without rpool */
	test_harness(NULL);
	return sm_test_end();
}
