/*
 * Copyright (c) 2006 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-aqrdq-0.c,v 1.3 2013/10/23 22:13:39 ca Exp $")
#define QMGR_DEBUG_DEFINE 1
#include "sm/common.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/qmgr-int.h"
#include "sm/actdb-int.h"
#include "sm/aqrdq.h"
#include "sm/test.h"

sm_str_P qmgr_hostname;
#define RCPT_LEN_MAX 256
#define SM_AQ_RCPTS 256
int Verbose;
sm_cstr_P Tst_CDB_id;

/*
**  TST_FILL_AQ -- fill AQ with "test" transactions
**
**	Parameters:
**		aq_ctx -- AQ context
**
**	Returns:
**		usual sm_error code
**
**	Last code review:
**	Last code change:
*/

sm_ret_T
tst_fill_aq(aq_ctx_P aq_ctx, int q_cnf_once_tas)
{
	sm_ret_T ret;
	int i;
	time_T time_now;
	aq_rcpt_P aq_rcpt;
	aq_ta_P aq_ta;
	static ulong added = 0;
	extern sm_cstr_P Tst_CDB_id;

	SM_IS_AQ(aq_ctx);
	aq_rcpt = NULL;
	aq_ta = NULL;

	time_now = 0;
	for (i = 0; i <q_cnf_once_tas; i++) {

		ret = aq_ta_add_new(aq_ctx, &aq_ta, 0, 1, THR_LOCK_UNLOCK);
		SM_TEST(sm_is_success(ret));
		if (sm_is_err(ret)) {
			sm_io_fprintf(smioerr,
				"sev=ERROR, func=qm_tst_fill_aq, aq_ta_add_new=%m"
				, ret);
			goto error;
		}

		ret = aq_rcpt_add_new(aq_ctx, aq_ta, &aq_rcpt,
				0 /*AQR_FL_IQDB*/,
				THR_LOCK_UNLOCK);
		SM_TEST(sm_is_success(ret));
		if (sm_is_err(ret)) {
			sm_io_fprintf(smioerr,
				"sev=ERROR, func=qm_tst_fill_aq, aq_rcpt_add_new=%m"
				, ret);
			goto error;
		}

		AQR_DA_INIT(aq_rcpt);
		aq_rcpt->aqr_entered = time_now;

		aq_ta->aqt_st_time = time_now;
		aq_ta->aqt_rcpts_inaq = 1;
		aq_ta->aqt_rcpts_tot = 1;
		aq_ta->aqt_msg_sz_b = 1;
		aq_ta->aqt_nxt_idx = 1;
		aq_ta->aqt_rcpts_left = 1;

		sm_snprintf(aq_ta->aqt_ss_ta_id, SMTP_STID_SIZE,
			SMTPS_STID_FORMAT,
			(id_count_T)added, 0);

		aq_ta->aqt_cdb_id = SM_CSTR_DUP(Tst_CDB_id);
#define TST_MAIL	"<s@example.com>"
		aq_ta->aqt_mail->aqm_pa = sm_str_scpy0(NULL, TST_MAIL,
						sizeof(TST_MAIL) + 4);
		if (NULL == aq_ta->aqt_mail->aqm_pa)
			goto error;
		ret = aq_tht_add(aq_ctx, aq_ta);
		if (sm_is_err(ret))
			goto error;

		AQR_SS_INIT(aq_rcpt);
		AQR_DA_INIT(aq_rcpt);
		SESSTA_COPY(aq_rcpt->aqr_ss_ta_id, aq_ta->aqt_ss_ta_id);
#define TST_RCPT	"<r@example.com>"
		aq_rcpt->aqr_pa = sm_str_scpy0(NULL, TST_RCPT,
						sizeof(TST_RCPT) + 4);
		if (NULL == aq_rcpt->aqr_pa)
			goto error;
		ret = aq_rht_add(aq_ctx, aq_rcpt);
		if (sm_is_err(ret))
			goto error;

		aq_rcpt->aqr_status = AQR_ST_NEW;
		aq_rcpt->aqr_addr_max = 1;
		aq_rcpt->aqr_addr_cur = 0;
		aq_rcpt->aqr_addrs = (aq_raddr_P) sm_malloc(sizeof(*(aq_rcpt->aqr_addrs)));
		if (NULL == aq_rcpt->aqr_addrs)
			goto error;
		IP_ASSIGN_IPV4(aq_rcpt->aqr_addrs[0].aqra_ip, htonl(INADDR_LOOPBACK));
		aq_rcpt->aqr_addrs[0].aqra_expt = time_now + 3600;
		aq_rcpt->aqr_addrs[0].aqra_pref = 0;
		aq_rcpt->aqr_entered = time_now;
		aq_rcpt->aqr_expire = time_now + 3600;
		aq_rcpt->aqr_flags = AQR_FL_RDY4DLVRY /*|AQR_FL_IQDB */;

		/* ignore result, a failure can be tolerated */
		(void) aq_rcpt_set_domain(aq_rcpt, qmgr_hostname);

		/* add rcpt to todo queue if address is available */
		ret = aq_rdq_add(aq_ctx, aq_rcpt, NULL, THR_LOCK_UNLOCK);
		SM_TEST(sm_is_success(ret));
		if (sm_is_err(ret)) {
			sm_io_fprintf(smioerr,
				"sev=ERROR, func=qm_tst_fill_aq, ss_ta=%s, rcpt_pa=%S, stat=%r, aq_rdq_add=%m"
				, aq_rcpt->aqr_ss_ta_id
				, aq_rcpt->aqr_pa
				, aq_rcpt->aqr_status, ret);
			goto error;
		}

		/* Reset pointers to avoid accidental free */
		aq_ta = NULL;
		aq_rcpt = NULL;
		++added;
	}

	if (aq_rcpt != NULL) {
		(void) aq_rcpt_rm(aq_ctx, aq_rcpt, AQR_RM_LOCK);
		aq_rcpt = NULL;
	}
	if (aq_ta != NULL) {
		(void) aq_ta_rm(aq_ctx, aq_ta, true);
		aq_ta = NULL;	/* not really necessary */
	}

sm_io_fprintf(smioerr,
"sev=INFO, func=qm_tst_fill_aq, added=%ul\n", added);

	return 0;

  error:
	if (aq_rcpt != NULL) {
		(void) aq_rcpt_rm(aq_ctx, aq_rcpt, AQR_RM_LOCK);
		aq_rcpt = NULL;	/* not really necessary */
	}
	if (aq_ta != NULL) {
		(void) aq_ta_rm(aq_ctx, aq_ta, true);
		aq_ta = NULL;	/* not really necessary */
	}
	return ret;
}

static sm_ret_T
aqr_test(uint lim)
{
	sm_ret_T ret;
	aq_ctx_P aq_ctx;
	sessta_id_T ta_id;

	aq_ctx = NULL;
	sm_snprintf(ta_id, SMTP_STID_SIZE, SMTPS_STID_FORMAT, (id_count_T)0, 0);
	Tst_CDB_id = sm_cstr_scpyn((uchar *)ta_id, sizeof(ta_id));
	if (NULL == Tst_CDB_id)
		return sm_error_temp(SM_EM_Q, ENOMEM);
	ret = aq_open(NULL /*qmgr_ctx*/, &aq_ctx, lim * 2, 0);

	SM_TEST_E(sm_is_success(ret));
	tst_fill_aq(aq_ctx, lim);

	return ret;

  error:
	return ret;
}

int
main(int argc, char **argv)
{
	int c;
	uint n;
	sm_ret_T r;
	char *prg;

	prg = argv[0];
	n = SM_AQ_RCPTS;
	while ((c = getopt(argc, argv, "V")) != -1) {
		switch (c) {
		  case 'V':
			++Verbose;
			break;
		  default:
			return -1;
		}
	}

#define MYHN "host.meta1.org"
	qmgr_hostname = sm_str_scpyn0(NULL, MYHN, sizeof(MYHN) + 1, 256);
	sm_test_begin(argc, argv, "test aqrdq 0");
	r = aqr_test(n);
	return sm_test_end();
}
