/*
 * Copyright (c) 2002-2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbputv.c,v 1.35 2011/01/30 21:49:05 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/varargs.h"
#include "sm/strrcb.h"
#include "sm/rcb.h"
#include "sm/reccom.h"
#include "sm/str-int.h"
#if !SM_NO_CSTR
# include "sm/cstr.h"
#endif
#include "sm/net.h"

/*
**  SM_RCB_PUTV -- Append a copy of items to an RCB
**
**	Parameters:
**		rcb -- RCB to append onto.
**		flags -- open_enc, put initial size, close_enc?
**		... -- data
**
**	Returns:
**		usual sm_error code; ENOMEM, SM_E_OVFLW_SC, SM_E_OVFLW_NS,
**			EINVAL, etc?
**
**	Side Effects: does not clean up rcb in case of an error.
**
**	Last code review: 2005-03-23 02:26:45
**	Last code change:
*/

sm_ret_T
sm_rcb_putv(sm_rcb_P rcb, uint flags, ...)
{
	int k;
	uint32_t rt, v;
	sm_str_P str;
#if !SM_NO_CSTR
	sm_cstr_P cstr;
#endif
	uchar *buf;
	uint len;
	sm_ret_T ret;
	sm_sockaddr_P sockaddr;
	sm_ipaddr_P ipaddr;
	va_list ap;

	SM_IS_RCB(rcb);
#if SM_RCB_CHECK
	SM_REQUIRE(SM_RCB_ENC == rcb->sm_rcb_state);
#endif

	if (SM_IS_FLAG(flags, RCB_PUTV_FIRST)) {
		/* encode rcb: placeholder for size */
		ret = sm_rcb_putuint32(rcb, 0);
		if (sm_is_err(ret))
			goto err2;
	}

	va_start(ap, flags);
#include "rcbputm.h"
	va_end(ap);

	return SM_SUCCESS;

  err1:
	va_end(ap);

  err2:
	return ret;
}
