#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm_inet_a2sockaddr.c,v 1.2 2013/12/27 17:42:39 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/net.h"

/*
**  SM_INET_A2SOCKADDR -- (check and) convert ASCII version of IP address to binary
**
**	Parameters:
**		cp -- begin of ASCII representation
**		end -- (pointer to) end of ASCII representation (output)
**		sockaddr -- sockaddr (output)
**
**	Returns:
**		usual sm_error code; ERANGE, EINVAL, ...
*/

sm_ret_T
sm_inet_a2sockaddr(const char *cp, char **end, sm_sockaddr_P sockaddr)
{
	sm_ret_T ret;

	SM_REQUIRE(cp != NULL);
	SM_REQUIRE(sockaddr != NULL);

	sm_memzero(sockaddr, sizeof(*sockaddr));
	ret = sm_inet_a2ipv4(cp, end, (ipv4_T *)&(sockaddr->sin.sin_addr));
	if (sm_is_ok(ret)) {
		sockaddr->sa.sa_family = AF_INET;
		return ret;
	}
#if MTA_NETINET6
	ret = sm_inet_a2ipv6(cp, end, (ipv6_T *)&(sockaddr->sin6.sin6_addr));
	if (sm_is_ok(ret)) {
		sockaddr->sa.sa_family = AF_INET6;
		return ret;
	}
#endif
	return sm_error_perm(SM_EM_IP, EINVAL);
}
