/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: pmilter_setmaclist.c,v 1.9 2013/10/23 22:13:53 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/reccom.h"
#include "pmilter.h"
#include "sm/pmfdef.h"
#include "sm/pmfapi.h"
#include "util.h"

#if MTA_USE_PMILTER

/*
**  SM_PMFI_SETMACS -- Set macros to be sent by MTA via array
**
**	Parameters:
**		pmss_ctx -- task context pmilter/SMTP server
**		where -- which stage?
**		macros -- array of macros
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmfi_setmacs(pmss_ctx_P pmss_ctx, uint where, uint32_t macros[])
{
	uint i;

	SM_IS_PMSS_CTX(pmss_ctx);
	if (where >= PM_SMST_MAX)
		return sm_err_perm(EINVAL);
	if (pmss_ctx->pmss_mac_names[where][0] != PMM_END)
		return sm_err_perm(EEXIST);

	for (i = 0; i < PM_MACROS_MAX && macros[i] != PMM_END; i++)
			;

	if (i >= PM_MACROS_MAX)
		return sm_err_perm(SM_E_OVFLW_NS);

	for (i = 0; i < PM_MACROS_MAX && macros[i] != PMM_END; i++) {
		pmss_ctx->pmss_mac_names[where][i] = macros[i];
	}

	return SM_SUCCESS;
}

/*
**  SM_PMFI_SETMACLIST -- Set list of macros to be sent by MTA
**
**	Parameters:
**		pmss_ctx -- task context pmilter/SMTP server
**		where -- which stage?
**		... -- list of macros
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmfi_setmaclist(pmss_ctx_P pmss_ctx, uint where, ...)
{
	uint i;
	uint32_t macros[PM_MACROS_MAX+1];
	uint32_t macro;
	va_list ap;

	va_start(ap, where);
	i = 0;
	while ((macro = va_arg(ap, uint32_t)) != PMM_END) {
		if (i >= PM_MACROS_MAX)
			return sm_err_perm(SM_E_OVFLW_NS);
		macros[i++] = macro;
	}
	va_end(ap);
	macros[i] = PMM_END;
	return sm_pmfi_setmacs(pmss_ctx, where, macros);
}
#endif /* MTA_USE_PMILTER */
