/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-pmilter.c,v 1.10 2014/03/25 18:42:08 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/mta.h"
#include "t-pmilter.h"

#if MTA_USE_PMILTER

sm_ret_T
pmtsectx_new(pmse_ctx_P pmse_ctx)
{
	pmtse_ctx_P pmtse_ctx;

	pmtse_ctx = (pmtse_ctx_P) malloc(sizeof(*pmtse_ctx));
	if (pmtse_ctx == NULL)
		return SMTP_R_TEMP;
	pmt_rcpts_init(pmtse_ctx);
	sm_pmfi_set_ctx_se(pmse_ctx, pmtse_ctx);
	return SM_SUCCESS;
}

sm_ret_T
pmtsectx_free(pmse_ctx_P pmse_ctx)
{
	pmtse_ctx_P pmtse_ctx;

	pmtse_ctx = (pmtse_ctx_P) sm_pmfi_get_ctx_se(pmse_ctx);
	if (pmtse_ctx != NULL) {
		pmt_rcpts_free(pmtse_ctx);
		free(pmtse_ctx);
	}
	sm_pmfi_set_ctx_se(pmse_ctx, NULL);
	return SM_SUCCESS;
}

/*
**  SM_GETSTAGE -- translate stage char into array index
**
**	Parameters:
**		arg -- stage (textual description)
**
**	Returns:
**		>=0: stage
**		<0 on error
*/

int
sm_getstage(char arg)
{
	int stage;

	switch (arg) {
	  case 'c':
		stage = SM_STAGE_NSEID;
		break;
	  case 'h':
	  case 'e':
		stage = SM_STAGE_HELO;
		break;
	  case 'm':
		stage = SM_STAGE_MAIL;
		break;
	  case 'r':
		stage = SM_STAGE_RCPT;
		break;
	  case 'D':
		stage = SM_STAGE_RSAD;
		break;
	  case 'd':
		stage = SM_STAGE_DATA;
		break;
	  case 'B':
		stage = SM_STAGE_MSG;
		break;
	  case 'b':
		stage = SM_STAGE_DOT;
		break;
	  case 'R':
		stage = SM_STAGE_RPLC_MSG;
		break;
	  default:
		stage = -1;
		break;
	}
	return stage;
}

#endif /* MTA_USE_PMILTER */
